import { Router } from "express";
import Database from "../db/database";
import authenticateJWT from "../lib/authenticateJWT";
import Word from "../types/Word";

const db = new Database();
const router = Router();

const times: { start: number, userId: number }[] = [];

const mapMultipleWords = async (words: Word[]) => {
    const categories = await db.query('SELECT * FROM categories', []);
    // console.log(categories)

    return words.map((word) => {
        if(word.japaneseWord.includes("|")) {
            const japaneseWords = word.japaneseWord.split("|");
            const romajiWords = word.romaji.split("|");
            return {
                wordID: word.wordID,
                polishWord: word.polishWord,
                japaneseWord: japaneseWords,
                romaji: romajiWords,
                category: {
                    categoryId: word.categoryId,
                    categoryEnglishName: categories.find((category: any) => category.categoryId === word.categoryId).englishName,
                    categoryPolishName: categories.find((category: any) => category.categoryId === word.categoryId).polishName
                }
            }
        }
        else return {
            wordID: word.wordID,
            polishWord: word.polishWord,
            japaneseWord: word.japaneseWord,
            romaji: word.romaji,
            category: {
                categoryId: word.categoryId,
                categoryEnglishName: categories.find((category: any) => category.categoryId === word.categoryId).englishName,
                categoryPolishName: categories.find((category: any) => category.categoryId === word.categoryId).polishName
            }
        
        };
    });

}

router.get('/all', (req, res) => {
    db.query('SELECT * FROM words', [])
        .then(async (result: Word[]) => {           
            res.json({ code: 200, words: await mapMultipleWords(result) });
        })
        .catch((err: any) => {
            console.log(err);
            res.sendStatus(500);
        });
});

router.get('/id/:id', (req, res) => {
    db.query('SELECT * FROM words WHERE wordId = ?', [req.params.id])
        .then(async (result: Word[]) => {
            if(result.length === 0) return res.status(404).json({ code: 404, error: 'Word not found' });
            res.json({ code: 200, word: await mapMultipleWords(result) });
        })
        .catch((err: any) => {
            console.log(err);
            res.status(500).json({ code: 500, error: 'Internal server error' });
        });
});

router.get('/category/:categoryId', (req, res) => {
    db.query('SELECT * FROM words WHERE categoryId = ?', [req.params.categoryId])
        .then(async (result: Word[]) => {
            if(result.length === 0) return res.status(404).json({ code: 404, error: 'Category not found' });    
            res.json({ code: 200, words: await mapMultipleWords(result) });
        })
        .catch((err: any) => {
            console.log(err);
            res.status(500).json({ code: 500, error: 'Internal server error' });
        });
});

router.get('/pickRandom/', (req, res) => {
    let { categoryId, amount } = req.query;

    if(!amount) amount = "1";
    
    const sql = `SELECT * FROM words ${categoryId ? `WHERE categoryId = ? ` : ""}ORDER BY RAND() LIMIT ${amount}`;

    db.query(sql, [categoryId])
        .then(async (result: any) => {
            if(result.length === 0) return res.status(404).json({ code: 404, error: 'Category not found' });
            res.json({ code: 200, words: await mapMultipleWords(result) });
        })
        .catch((err: any) => {
            console.log(err);
            res.status(500).json({ code: 500, error: 'Internal server error' });
        });
});

router.post('/startTest', authenticateJWT, async (req, res) => {
    times.push({
        start: Date.now(),
        // @ts-ignore
        userId: req.user.userId
    })

    return res.json({ code: 200, message: 'Test started' });
});

router.post('/submitAnswer', authenticateJWT, (req, res) => {
    const answers = req.body as { id: number, answer: string }[];
    const promises = answers.map(answer => db.query('SELECT * FROM words WHERE wordId = ?', [answer.id]));
    
    Promise.all(promises)
        .then(async (results: Word[][]) => {
            const verified = results.map((result, index) => {
                if(result[0].romaji.includes("|")) {
                    const romajiWords = result[0].romaji.split("|");
                    return {id: result[0].wordID, correct: romajiWords.includes(answers[index].answer) }
                }
                return {id: result[0].wordID, correct: result[0].romaji === answers[index].answer }
            });

            const accuracy = verified.filter((v) => v.correct).length / verified.length;
            const xp = accuracy * 10 + (accuracy === 1 ? 10 : 0);

            // @ts-ignore
            const time = times.find((time) => time.userId === req.user.userId);
            if(!time) return res.status(404).json({ code: 404, error: 'Test not started' });

            times.splice(times.indexOf(time), 1);

            // @ts-ignore
            await db.updateUserXP(req.user.userId, xp);

            // @ts-ignore
            await db.addScore(req.user.userId, Math.floor((Date.now() - time.start) / 1000), xp, verified.filter((v) => !v.correct).length);

            res.json({ code: 200, results: {
                questions: verified,
                timeElapsed: Math.floor((Date.now() - time.start) / 1000),
                accuracy,
                experienceEarned: xp
            }});
        })
        .catch((err: any) => {
            console.log(err);
            res.status(500).json({ code: 500, error: 'Internal server error' });
        });
});

router.post('/submitPolishAnswer', authenticateJWT, (req, res) => {
    const answers = req.body as { id: number, answer: string }[];
    const promises = answers.map(answer => db.query('SELECT * FROM words WHERE wordId = ?', [answer.id]));
    
    Promise.all(promises)
        .then(async (results: Word[][]) => {
            const verified = results.map((result, index) => {
                return {id: result[0].wordID, correct: result[0].polishWord === answers[index].answer }
            });

            const accuracy = verified.filter((v) => v.correct).length / verified.length;
            const xp = accuracy * 10 + (accuracy === 1 ? 10 : 0);

            // @ts-ignore
            const time = times.find((time) => time.userId === req.user.userId);
            if(!time) return res.status(404).json({ code: 404, error: 'Test not started' });

            times.splice(times.indexOf(time), 1);

            // @ts-ignore
            await db.updateUserXP(req.user.userId, xp);

            // @ts-ignore
            await db.addScore(req.user.userId, Math.floor((Date.now() - time.start) / 1000), xp, verified.filter((v) => !v.correct).length);

            res.json({ code: 200, results: {
                questions: verified,
                timeElapsed: Math.floor((Date.now() - time.start) / 1000),
                accuracy,
                experienceEarned: xp
            }});
        })
        .catch((err: any) => {
            console.log(err);
            res.status(500).json({ code: 500, error: 'Internal server error' });
        });
});

export default router;